@@ -8,7 +8,7 @@ use warnings;
 use vars qw($VERSION @EXPORT_OK %EXPORT_TAGS $DEBUG);
 
 $DEBUG = 0;
-$VERSION = '0.36';
+$VERSION = '0.38';
 
 use base qw(Class::Accessor);
 XML::Sig->mk_accessors(qw(key));
@@ -27,7 +27,7 @@ use Carp;
 
 
 use constant TRANSFORM_ENV_SIG           => 'http://www.w3.org/2000/09/xmldsig#enveloped-signature';
-use constant TRANSFORM_C14N              => 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315#';
+use constant TRANSFORM_C14N              => 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315';
 use constant TRANSFORM_C14N_COMMENTS     => 'http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments';
 use constant TRANSFORM_C14N_V1_1         => 'http://www.w3.org/TR/2008/REC-xml-c14n11-20080502';
 use constant TRANSFORM_C14N_V1_1_COMMENTS => 'http://www.w3.org/TR/2008/REC-xml-c14n11-20080502#WithComments';
@@ -205,9 +205,6 @@ sub verify {
     while (my $signature_node = $signature_nodeset->shift()) {
         $i++;
         print ("\nSignature $i\n") if $DEBUG;
-        # set the namespace for the signature to a known prefix
-        $signature_node->setNamespace(
-            'http://www.w3.org/2000/09/xmldsig#', 'dsig', 1 );
 
         # Get SignedInfo Reference ID
         my $reference = $self->{ parser }->findvalue(
@@ -256,6 +253,7 @@ sub verify {
         my ($signed_info) = $self->{ parser }->findnodes('dsig:SignedInfo', $signature_node);
         my $signed_info_canon = $self->_canonicalize_xml($signed_info, $signature_node);
 
+        print "$signed_info_canon\n" if $DEBUG;
         if(Digest::SHA->can($digest_method)) {
             my $rsa_hash = "use_$digest_method" . "_hash";
             $self->{rsa_hash} =  "use_$digest_method" . "_hash";
@@ -445,26 +443,33 @@ sub _transform {
         $context
     );
 
+    print "_transform\n" if $DEBUG;
     foreach my $node ($transforms->get_nodelist) {
         my $alg = $node->getAttribute('Algorithm');
 
-        print "Algorithm: $alg\n" if $DEBUG;
+        print "    Algorithm: $alg\n" if $DEBUG;
         if ($alg eq TRANSFORM_ENV_SIG) {
             # TODO the xml being passed here currently has the
             # Signature removed.  May be better to do it all here
             next;
         }
         elsif ($alg eq TRANSFORM_C14N) {
-           $xml = $xml->toStringC14N();
+            print "        toStringC14N" if $DEBUG;
+            $xml = $xml->toStringC14N();
         }
         elsif ($alg eq TRANSFORM_C14N_COMMENTS) {
+            print "        toStringC14N(1)" if $DEBUG;
             $xml = $xml->toStringC14N(1);
         }
         elsif ($alg eq TRANSFORM_EXC_C14N) {
-            $xml = $xml->toStringEC14N();
+            my @prefixlist = $self->_find_prefixlist($node);
+            print "        toStringEC14N(0, '', @prefixlist)\n" if $DEBUG;
+            $xml = $xml->toStringEC14N(0, '', \@prefixlist);
         }
         elsif ($alg eq TRANSFORM_EXC_C14N_COMMENTS) {
-            $xml = $xml->toStringEC14N(1);
+            my @prefixlist = $self->_find_prefixlist($node);
+            print "        toStringEC14N(1, '', @prefixlist)\n" if $DEBUG;
+            $xml = $xml->toStringEC14N(1, '', \@prefixlist);
         }
         else {
             die "Unsupported transform: $alg";
@@ -473,6 +478,31 @@ sub _transform {
     return $xml;
 }
 
+##
+## _find_prefixlist($node)
+##
+## Arguments:
+##    $node:    string XML NodeSet
+##
+## Returns: ARRAY of prefix lists
+##
+## Generate an array of prefix lists defined in InclusiveNamespaces
+##
+sub _find_prefixlist {
+    my $self = shift;
+    my ($node) = @_;
+    my @children = $node->getChildrenByTagName('InclusiveNamespaces');
+
+    my $prefixlist = '';
+    foreach my $child (@children) {
+        if ($child) {
+            $prefixlist .= $child->getAttribute('PrefixList');
+        }
+        $prefixlist .= ' ';
+    }
+    return split / /, $prefixlist;
+}
+
 ##
 ## _verify_rsa($context,$canonical,$sig)
 ##
@@ -1171,7 +1201,7 @@ XML::Sig
 
 =head1 VERSION
 
-version 0.36
+version 0.38
 
 =head1 SYNOPSIS
 
